#ifndef FHGFSOPSINODE_H_
#define FHGFSOPSINODE_H_

#include <app/App.h>
#include <common/threading/Mutex.h>
#include <common/toolkit/Time.h>
#include <net/filesystem/FhgfsOpsRemoting.h>
#include <filesystem/FsDirInfo.h>
#include <filesystem/FsFileInfo.h>
#include <toolkit/NoAllocBufferStore.h>
#include <common/Common.h>
#include "FhgfsInode.h"
#include "FhgfsOps_versions.h"

#include <linux/fs.h>
#include <linux/vfs.h>


// forward declaration
struct App;

struct FhgfsInodeComparisonInfo;
typedef struct FhgfsInodeComparisonInfo FhgfsInodeComparisonInfo;


#ifndef KERNEL_HAS_ATOMIC_OPEN
   extern struct dentry* FhgfsOps_lookupIntent(struct inode* parentDir, struct dentry* dentry,
      struct nameidata* nameidata);
#else
   extern struct dentry* FhgfsOps_lookupIntent(struct inode* parentDir, struct dentry* dentry,
      unsigned flags);
#endif // LINUX_VERSION_CODE

extern int FhgfsOps_getattr(struct vfsmount* mnt, struct dentry* dentry, struct kstat* kstat);
extern int FhgfsOps_setattr(struct dentry* dentry, struct iattr* iattr);

#ifdef KERNEL_HAS_UMODE_T
extern int FhgfsOps_mkdir(struct inode* dir, struct dentry* dentry, umode_t mode);
extern int FhgfsOps_mknod(struct inode* dir, struct dentry* dentry, umode_t mode, dev_t dev);
#else
extern int FhgfsOps_mkdir(struct inode* dir, struct dentry* dentry, int mode);
extern int FhgfsOps_mknod(struct inode* dir, struct dentry* dentry, int mode, dev_t dev);
#endif // KERNEL_HAS_UMODE_T

#if defined KERNEL_HAS_ATOMIC_OPEN
   int FhgfsOps_atomicOpen(struct inode* dir, struct dentry* dentry, struct file* file,
      unsigned openFlags, umode_t createMode,  int* outOpenedFlags);
   extern int FhgfsOps_createIntent(struct inode* dir, struct dentry* dentry, umode_t mode,
      bool isExclusiveCreate);
#elif defined KERNEL_HAS_UMODE_T
   extern int FhgfsOps_createIntent(struct inode* dir, struct dentry* dentry, umode_t mode,
      struct nameidata* nameidata);
#else
   extern int FhgfsOps_createIntent(struct inode* dir, struct dentry* dentry, int mode,
      struct nameidata* nameidata);
#endif // KERNEL_HAS_ATOMIC_OPEN

extern int FhgfsOps_rmdir(struct inode* dir, struct dentry* dentry);
extern int FhgfsOps_unlink(struct inode* dir, struct dentry* dentry);

extern int FhgfsOps_symlink(struct inode* dir, struct dentry* dentry, const char* to);

extern int FhgfsOps_link(struct dentry* dentryFrom, struct inode* inode, struct dentry* dentryTo);
extern int FhgfsOps_hardlinkAsSymlink(struct dentry* oldDentry, struct inode* dir,
   struct dentry* newDentry);


extern void* FhgfsOps_follow_link(struct dentry* dentry, struct nameidata* nd);
extern void FhgfsOps_put_link(struct dentry *dentry, struct nameidata* nd, void* p);

extern int FhgfsOps_rename(struct inode* inodeDirFrom, struct dentry* dentryFrom,
   struct inode* inodeDirTo, struct dentry* dentryTo);

#if LINUX_VERSION_CODE < KERNEL_VERSION(3,8,0)
extern void FhgfsOps_truncate(struct inode* inode);
#endif // LINUX_VERSION_CODE

extern int FhgfsOps_vmtruncate(struct inode* inode, loff_t offset);


extern fhgfs_bool FhgfsOps_initInodeCache(void);
extern void FhgfsOps_destroyInodeCache(void);
extern struct inode* FhgfsOps_alloc_inode(struct super_block* sb);
extern void FhgfsOps_destroy_inode(struct inode* inode);

extern struct inode* __FhgfsOps_newInodeWithParentID(struct super_block* sb, struct kstat* kstat,
   dev_t dev, EntryInfo* entryInfo, uint16_t parentNodeID, FhgfsIsizeHints* iSizeHints);

extern int __FhgfsOps_instantiateInode(struct dentry* dentry, EntryInfo* entryInfo,
   fhgfs_stat* fhgfsStat, FhgfsIsizeHints* iSizeHints);
int __FhgfsOps_compareInodeID(struct inode* cachedInode, void* newInodeInfo);
int __FhgfsOps_initNewInodeDummy(struct inode* newInode, void* newInodeInfo);

static inline int __FhgfsOps_refreshInode(App* app, struct inode* inode, fhgfs_stat* fhgfsStat,
   FhgfsIsizeHints* iSizeHints);
extern int __FhgfsOps_doRefreshInode(App* app, struct inode* inode, fhgfs_stat* fhgfsStat,
   FhgfsIsizeHints* iSizeHints, fhgfs_bool noFlush);
extern int __FhgfsOps_revalidateMapping(App* app, struct inode* inode);
extern int __FhgfsOps_flushInodeFileCache(App* app, struct inode* inode);


// inliners
static inline void __FhgfsOps_applyStatDataToInode(struct kstat* kstat, FhgfsIsizeHints* iSizeHints,
   struct inode* outInode);
static inline void __FhgfsOps_applyStatDataToInodeUnlocked(struct kstat* kstat,
   FhgfsIsizeHints* iSizeHints, struct inode* outInode);
static inline void __FhgfsOps_applyStatAttribsToInode(struct kstat* kstat, struct inode* outInode);
static inline void __FhgfsOps_applyStatSizeToInode(struct kstat* kstat,
   FhgfsIsizeHints* iSizeHints, struct inode* inOutInode);
static inline struct inode* __FhgfsOps_newInode(struct super_block* sb, struct kstat* kstat,
   dev_t dev, EntryInfo* entryInfo,  FhgfsIsizeHints* iSizeHints);
static inline fhgfs_bool __FhgfsOps_isPagedMode(struct super_block* sb);

/**
 * This structure is passed to _compareInodeID().
 */
struct FhgfsInodeComparisonInfo
{
   ino_t inodeHash; // (=> inode::i_ino)
   const char* entryID;
};


/**
 * Note: acquires i_lock spinlock to protect i_size and i_blocks
 */
void __FhgfsOps_applyStatDataToInode(struct kstat* kstat, FhgfsIsizeHints* iSizeHints,
   struct inode* outInode)
{
   __FhgfsOps_applyStatAttribsToInode(kstat, outInode);

   spin_lock(&outInode->i_lock); // I _ L O C K

   __FhgfsOps_applyStatSizeToInode(kstat, iSizeHints, outInode);

   spin_unlock(&outInode->i_lock); // I _ U N L O C K
}

/**
 * Note: Caller must hold i_lock.
 */
void __FhgfsOps_applyStatDataToInodeUnlocked(struct kstat* kstat, FhgfsIsizeHints* iSizeHints,
   struct inode* outInode)
{
   __FhgfsOps_applyStatAttribsToInode(kstat, outInode);

   __FhgfsOps_applyStatSizeToInode(kstat, iSizeHints, outInode);
}

/**
 * Note: Don't call this directly - use the _applyStatDataToInode... wrappers.
 */
void __FhgfsOps_applyStatAttribsToInode(struct kstat* kstat, struct inode* outInode)
{
   App* app = FhgfsOps_getApp(outInode->i_sb);
   Config* cfg = App_getConfig(app);

   // remote attribs (received from nodes)
   outInode->i_mode = kstat->mode;
   outInode->i_uid = kstat->uid;
   outInode->i_gid = kstat->gid;
   outInode->i_atime = kstat->atime;
   outInode->i_mtime = kstat->mtime;
   outInode->i_ctime = kstat->ctime;

   set_nlink(outInode, kstat->nlink);

   outInode->i_blkbits = Config_getTuneInodeBlockBits(cfg);
}

/**
 * Note: Don't call this directly - use the _applyStatDataToInode... wrappers.
 * Note: Caller must hold i_lock.
 *
 * @param iSizeHints might be NULL
 */
void __FhgfsOps_applyStatSizeToInode(struct kstat* kstat, FhgfsIsizeHints* iSizeHints,
   struct inode* inOutInode)
{
   FhgfsInode* fhgfsInode = FHGFS_INODE(inOutInode);

   if (S_ISREG(inOutInode->i_mode) && FhgfsInode_getHasPageWriteFlag(fhgfsInode) )
   {
      loff_t oldSize = i_size_read(inOutInode);
      uint64_t lastWriteBackOrIsizeWriteTime;

      /* We don't allow to decrease the file size in paged mode, as
       * this may/will cause data corruption (zeros/holes instead of real data).
       * The detection when to apply the server i_size is rather complex. */
      if (oldSize > kstat->size)
      {
         if (FhgfsInode_getNoIsizeDecrease(fhgfsInode) )
            return;

         // there are current write-back threads running
         if (FhgfsInode_getWriteBackCounter(fhgfsInode) )
            return;

         // must be read after reading the write-back counter
         lastWriteBackOrIsizeWriteTime = FhgfsInode_getLastWriteBackOrIsizeWriteTime(fhgfsInode);

         if (iSizeHints)
         {
            if (iSizeHints->ignoreIsize)
               return; // iSizeHints tells us to ignore the value

            if (time_after_eq((unsigned long) lastWriteBackOrIsizeWriteTime,
                           (unsigned long) iSizeHints->timeBeforeRemoteStat) )
               return; /* i_size was updated or a writeback thread finished after we send the
                        * remote stat call, so concurrent stat/isize updates and we need to ignore
                          the remote value. */
         }

          #ifdef FHGFS_DEBUG
          {
             static fhgfs_bool didLog = fhgfs_false;

             struct super_block* sb = inOutInode->i_sb;
             App* app = FhgfsOps_getApp(sb);
             Logger* log = App_getLogger(app);

             if (!didLog)
             {
                // unlock, as the logger gets a mutex lock -> possible lock order issue
                spin_unlock(&inOutInode->i_lock);

                LOG_DEBUG_FORMATTED(log, Log_WARNING, __func__,
                   "Warn-once: (possibly valid) isize-shrink oldSize: %ld newSize: %ld "
                   "lastWBTime: %lu, timeBeforeRemote; %lu\n",
                   oldSize, kstat->size, lastWriteBackOrIsizeWriteTime, iSizeHints->timeBeforeRemoteStat);
                dump_stack();

                didLog = fhgfs_true;

                spin_lock(&inOutInode->i_lock);
             }
          }
          #endif
      }
   }

   i_size_write(inOutInode, kstat->size);
   inOutInode->i_blocks = kstat->blocks;
}

/**
 * See __FhgfsOps_newInodeWithParentID for details. This is just a wrapper function.
 */
struct inode* __FhgfsOps_newInode(struct super_block* sb, struct kstat* kstat, dev_t dev,
   EntryInfo* entryInfo,  FhgfsIsizeHints* iSizeHints)
{
      return __FhgfsOps_newInodeWithParentID(sb, kstat, dev, entryInfo, 0, iSizeHints);
}

fhgfs_bool __FhgfsOps_isPagedMode(struct super_block* sb)
{
   App* app = FhgfsOps_getApp(sb);
   Config* cfg = App_getConfig(app);

   if (Config_getTuneFileCacheTypeNum(cfg) == FILECACHETYPE_Paged)
      return fhgfs_true;

   return fhgfs_false;
}

/**
 * See __FhgfsOps_doRefreshInode for details.
 */
int __FhgfsOps_refreshInode(App* app, struct inode* inode, fhgfs_stat* fhgfsStat,
   FhgfsIsizeHints* iSizeHints)
{
   // do not disable inode flushing from this (default) refreshInode function
   return __FhgfsOps_doRefreshInode(app, inode, fhgfsStat, iSizeHints, fhgfs_false);
}



#endif /* FHGFSOPSINODE_H_ */
