#ifndef FSDIRINFO_H_
#define FSDIRINFO_H_

#include <common/Common.h>
#include <common/toolkit/vector/StrCpyVec.h>
#include <common/toolkit/vector/IntCpyVec.h>
#include "FsObjectInfo.h"


enum ListDirIncStage; // forward declaration
struct StrCpyVec; // forward declaration
struct IntCpyVec; // forward declaration


struct FsDirInfo;
typedef struct FsDirInfo FsDirInfo;

extern void FsDirInfo_init(FsDirInfo* this, App* app);
extern FsDirInfo* FsDirInfo_construct(App* app);
extern void FsDirInfo_uninit(FsObjectInfo* this);
extern void FsDirInfo_destruct(FsObjectInfo* this);

// getters & setters
extern loff_t FsDirInfo_getServerOffset(FsDirInfo* this);
extern void FsDirInfo_setServerOffset(FsDirInfo* this, int64_t serverOffset);
extern loff_t FsDirInfo_getIncrementalOffset(FsDirInfo* this);
extern void FsDirInfo_setIncrementalOffset(FsDirInfo* this, uint64_t incrementalOffset);
size_t FsDirInfo_getCurrentContentsPos(FsDirInfo* this);
void FsDirInfo_setCurrentContentsPos(FsDirInfo* this, size_t currentContentsPos);
extern struct StrCpyVec* FsDirInfo_getDirContents(FsDirInfo* this);
extern struct StrCpyVec* FsDirInfo_getEntryIDs(FsDirInfo* this);
extern struct IntCpyVec* FsDirInfo_getDirContentsTypes(FsDirInfo* this);
extern void FsDirInfo_setEndOfDir(FsDirInfo* this, fhgfs_bool endOfDir);
extern fhgfs_bool FsDirInfo_getEndOfDir(FsDirInfo* this);


struct FsDirInfo
{
   FsObjectInfo fsObjectInfo;

   StrCpyVec dirContents;
   IntCpyVec dirContentsTypes; // DirEntryType elements matching dirContents vector
   StrCpyVec entryIDs;
   int64_t serverOffset; // offset for the next incremental list request to the server
   uint64_t incrementalOffset; // seek to n-th elem offset; only used when serverOffset is -1;
   size_t currentContentsPos; // current local pos in dirContents (>=0 && <dirContents_len)
   fhgfs_bool endOfDir; // fhgfs_true if server reached end of dir entries during last query
};



#endif /*FSDIRINFO_H_*/
